"""
Convenience module for importing xwnode.

This allows users to import the library in two ways:
1. import exonware.xwnode
2. import xwnode  # This convenience import

Company: eXonware.com
Author: Eng. Muhammad AlShehri
Email: connect@exonware.com
Version: 0.0.1.15
Generation Date: 07-Sep-2025
"""

# Import everything from the main package
from exonware.xwnode import *  # noqa: F401, F403

# Preserve the version
__version__ = "0.0.1"